@@ -261,6 +261,9 @@ def check_backwards_compatibility(codec_id, arrays, codecs, precision=None, pref
 
         for j, codec in enumerate(codecs):
 
+            if codec is None:
+                pytest.skip("codec has been removed")
+
             # setup a directory to hold encoded data
             codec_dir = os.path.join(fixture_dir, 'codec.{:02d}'.format(j))
             if not os.path.exists(codec_dir):  # pragma: no cover
@@ -268,14 +271,12 @@ def check_backwards_compatibility(codec_id, arrays, codecs, precision=None, pref
 
             # file with codec configuration information
             codec_fn = os.path.join(codec_dir, 'config.json')
-
             # one time save config
             if not os.path.exists(codec_fn):  # pragma: no cover
                 with open(codec_fn, mode='w') as cf:
                     _json.dump(codec.get_config(), cf, sort_keys=True, indent=4)
-
             # load config and compare with expectation
-            with open(codec_fn, mode='r') as cf:
+            with open(codec_fn) as cf:
                 config = _json.load(cf)
                 assert codec == get_codec(config)
 
@@ -320,7 +321,7 @@ def check_err_encode_object_buffer(compressor):
 
 
 def check_max_buffer_size(codec):
-    for max_buffer_size in [4, 64, 1024]:
+    for max_buffer_size in (4, 64, 1024):
         old_max_buffer_size = codec.max_buffer_size
         try:
             codec.max_buffer_size = max_buffer_size
