@@ -2,6 +2,7 @@ import unittest
 
 
 import numpy as np
+import pytest
 
 
 try:
@@ -52,30 +53,32 @@ def test_backwards_compatibility():
     check_backwards_compatibility(codec.codec_id, arrays, [codec])
 
 
-def test_non_numpy_inputs():
+@pytest.mark.parametrize(
+    "input_data, dtype",
+    [
+        ([0, 1], None),
+        ([[0, 1], [2, 3]], None),
+        ([[0], [1], [2, 3]], object),
+        ([[[0, 0]], [[1, 1]], [[2, 3]]], None),
+        (["1"], None),
+        (["11", "11"], None),
+        (["11", "1", "1"], None),
+        ([{}], None),
+        ([{"key": "value"}, ["list", "of", "strings"]], object),
+        ([b"1"], None),
+        ([b"11", b"11"], None),
+        ([b"11", b"1", b"1"], None),
+        ([{b"key": b"value"}, [b"list", b"of", b"strings"]], object),
+    ]
+)
+def test_non_numpy_inputs(input_data, dtype):
     codec = MsgPack()
     # numpy will infer a range of different shapes and dtypes for these inputs.
     # Make sure that round-tripping through encode preserves this.
-    data = [
-        [0, 1],
-        [[0, 1], [2, 3]],
-        [[0], [1], [2, 3]],
-        [[[0, 0]], [[1, 1]], [[2, 3]]],
-        ["1"],
-        ["11", "11"],
-        ["11", "1", "1"],
-        [{}],
-        [{"key": "value"}, ["list", "of", "strings"]],
-        [b"1"],
-        [b"11", b"11"],
-        [b"11", b"1", b"1"],
-        [{b"key": b"value"}, [b"list", b"of", b"strings"]],
-    ]
-    for input_data in data:
-        actual = codec.decode(codec.encode(input_data))
-        expect = np.array(input_data)
-        assert expect.shape == actual.shape
-        assert np.array_equal(expect, actual)
+    actual = codec.decode(codec.encode(input_data))
+    expect = np.array(input_data, dtype=dtype)
+    assert expect.shape == actual.shape
+    assert np.array_equal(expect, actual)
 
 
 def test_encode_decode_shape_dtype_preserved():
