@@ -1,8 +1,8 @@
+from contextlib import suppress
+
 _zfpy = None
-try:
+with suppress(ImportError):
     import zfpy as _zfpy
-except ImportError:  # pragma: no cover
-    pass
 
 
 if _zfpy:
@@ -46,8 +46,6 @@ if _zfpy:
                 self.compression_kwargs = {"rate": rate}
             elif mode == _zfpy.mode_fixed_precision:
                 self.compression_kwargs = {"precision": precision}
-            else:
-                pass
 
             self.tolerance = tolerance
             self.rate = rate
